package controller;

import java.util.List;
import java.util.Scanner;

import javax.swing.SwingUtilities;

import com.jamonapi.Monitor;

/**
 * 
 * Controller-Klasse des MVC-Patterns
 * 
 * Einfhrung in den SQL-Dialekt JPQL von JPA siehe hier:
 * http://www.objectdb.com/java/jpa/query
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class Controller {
	public static Monitor performanceMonitor = null;

	/**
	 * parameterloser Konstruktor
	 */
	public Controller() {
		ModelController.connectDatabase();

		/**
		 * Im Connection-String zu JPA steht zwar "shutdown=true" so dass die
		 * Lock-Datei der Datenbank beim Trennen der einzig zulssigen
		 * Verbindung gelscht wird (wie bei Embedded-Datenbanken blich), aber
		 * dazu muss die Anwendung schon korrekt beendet werden damit der
		 * EntityManager und die EntityManagerFactory korrekt geschlossen
		 * werden.
		 * 
		 * Sprich Strg-C im Konsolenfenster muss abgefangen werden.
		 */
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				/**
				 * Clients benachrichtigen und beenden - sollte moeglichst
				 * asynchron zur Beendigung des Serverprozesses laufen
				 */
				System.out.println(""); //$NON-NLS-1$
				RmiServer.consolelogger.info(Messages.getString("Controller.1")); //$NON-NLS-1$
				ObserverUpdater.terminateClients();

				/**
				 * Server beenden
				 */
				RmiServer.consolelogger.info(Messages.getString("Controller.2")); //$NON-NLS-1$

				ModelController.disconnectDatabase();
			}
		});

		/**
		 * Auf Eingaben parallel reagieren
		 */
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				System.out.println(""); //$NON-NLS-1$
				System.out.print(Messages.getString("Controller.4")); //$NON-NLS-1$
				while (true) {
					Scanner scanner = new Scanner(System.in);
					if (scanner.hasNext()) {
						String input = scanner.nextLine();

						if (input.equals("q")) { //$NON-NLS-1$
							System.exit(0);
						} else {
							System.out.print(Messages.getString("Controller.6")); //$NON-NLS-1$
						}
					}
				}
			}
		});
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten und weiterreichen an die Beobachter
	 * (Clients)
	 * 
	 */
	public void selectData(Class tableName) {
		ModelController.selectData(tableName);
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten und direktes Zurckliefern ber eine
	 * Liste
	 * 
	 */
	public List<Object> getData(Class tableName) {
		return ModelController.getData(tableName);
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten mit WHERE-Bedingung und direktes
	 * Zurckliefern ber eine Liste
	 * 
	 */
	public List<Object> getDataWithClause(Class tableName, String column,
			String clause) {
		return ModelController.getDataWithClause(tableName, column, clause);
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten mit WHERE-Bedingung und direktes
	 * Zurckliefern ber eine Liste
	 * 
	 */
	public List<Object> getDataWithClause(Class tableName, String column,
			Integer clause) {
		return ModelController.getDataWithClause(tableName, column, clause);
	}

	/**
	 * Schreibt einen Datensatz in die Datenbank
	 * 
	 * @param type
	 *            Entity
	 * @return true wenn Speichern funktioniert hat
	 */
	public boolean insertData(Object type) {
		return ModelController.insertData(type);
	}

	/**
	 * ndert einen Datensatz in der Datenbank
	 * 
	 * @param entity
	 *            Entity
	 * @return true wenn Speichern funktioniert hat
	 */
	public boolean changeData(Object entity) {
		return ModelController.changeData(entity);
	}

	/**
	 * Lscht einen Datensatz in der Datenbank
	 * 
	 * @param entity
	 *            Entity
	 * @return true wenn Speichern funktioniert hat
	 */
	public boolean deleteData(Object entity) {
		return ModelController.deleteData(entity);
	}
}
