package controller;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.metamodel.Attribute;
import javax.persistence.metamodel.ManagedType;
import com.jamonapi.*;

/**
 * Utility-Klasse zur Ansteuerung der Datenbank
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class ModelController {
	public static EntityManagerFactory emf = null;
	public static EntityManager em = null;

	private static ArrayList<ManagedType<?>> tableList = null;

	/**
	 * privater Konstruktor, es knnen keine Instanzen gebildet werden
	 */
	private ModelController() {
	}

	/**
	 * Herstellen der Datenbankverbindung
	 */
	public static void connectDatabase() {
		emf = Persistence.createEntityManagerFactory("JPA"); //$NON-NLS-1$
		em = emf.createEntityManager();

		/**
		 * Einlesen der Entity-Klassen
		 */
		tableList = new ArrayList<ManagedType<?>>();
		for (ManagedType<?> table : em.getMetamodel().getManagedTypes()) {
			tableList.add(table);
		}

		/**
		 * referenzierte Tabellen auslesen - anhand eines Objektes -
		 * funktioniert so:
		 */
		/*
		 * for (Field field : type.getClass().getDeclaredFields()) { for (Object
		 * table : tableList) { if (field.getType().equals( ((ManagedType<?>)
		 * table).getJavaType())) {
		 * System.out.println(field.getType().getName());
		 * selectDataFromClass(field.getType()); } } }
		 */
	}

	/**
	 * Lsen der Datenbankverbindung
	 */
	public static void disconnectDatabase() {
		/*
		 * Ressourcen schlieen, Datenbankverbindung muss aber nicht explizit
		 * geschlossen werden weil "shutdown=true" im Connectionstring steht.
		 */
		if (emf.isOpen()) {
			emf.close();
		}
		if (em.isOpen()) {
			em.close();
		}
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten und weiterreichen an die Beobachter
	 * (Clients)
	 * 
	 */
	public static void selectData(Class table) {
		List<Object> results = getData(table);

		/**
		 * Views aktualisieren
		 */
		ObserverUpdater.update(table, results);
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten und direktes Zurckliefern ber eine
	 * Liste.
	 * 
	 * Da alle Datenstze aus der angegebenen Tabelle auf einmal ausgelesen
	 * werden werden Performance-Messungen mittels JAMon gemacht.
	 * 
	 */
	public static List<Object> getData(Class tableName) {
		/**
		 * Instanziieren und Starten des Performance-Monitors
		 */
		Controller.performanceMonitor = MonitorFactory.start(Messages
				.getString("ModelController.4") //$NON-NLS-1$
				+ tableName.getName());

		/**
		 * Auslesen der Daten
		 */
		List<Object> results = null;
		try {
			Query query = em.createQuery("SELECT C FROM " + tableName.getName() //$NON-NLS-1$
					+ " C"); //$NON-NLS-1$
			results = query.getResultList();
		} catch (Exception e) {
			System.out.println(""); //$NON-NLS-1$
			RmiServer.consolelogger.error(Messages
					.getString("ModelController.0") + e.getMessage()); //$NON-NLS-1$
		}

		/**
		 * Stoppen des Performance-Monitors
		 */
		Controller.performanceMonitor.stop();

		/**
		 * Performance-Messung loggen
		 */
		RmiServer.fileLogger.info(Controller.performanceMonitor);

		return results;
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten mit WHERE-Bedingung und direktes
	 * Zurckliefern ber eine Liste
	 * 
	 */
	public static List<Object> getDataWithClause(Class tableName,
			String column, String clause) {
		List<Object> results = null;
		try {
			Query query = em.createQuery("SELECT C FROM " + tableName.getName() //$NON-NLS-1$
					+ " C WHERE C." + column + "=:clause"); //$NON-NLS-1$ //$NON-NLS-2$
			query.setParameter("clause", clause); //$NON-NLS-1$
			results = query.getResultList();
		} catch (Exception e) {
			System.out.println(""); //$NON-NLS-1$
			RmiServer.consolelogger.error(Messages
					.getString("ModelController.0") + e.getMessage()); //$NON-NLS-1$
		}

		return results;
	}

	/**
	 * 
	 * Auslesen von spezifischen Daten mit WHERE-Bedingung und direktes
	 * Zurckliefern ber eine Liste
	 * 
	 */
	public static List<Object> getDataWithClause(Class tableName,
			String column, Integer clause) {
		List<Object> results = null;
		try {
			Query query = em.createQuery("SELECT C FROM " + tableName.getName() //$NON-NLS-1$
					+ " C WHERE C." + column + "=:clause"); //$NON-NLS-1$ //$NON-NLS-2$
			query.setParameter("clause", clause); //$NON-NLS-1$
			results = query.getResultList();
		} catch (Exception e) {
			System.out.println(""); //$NON-NLS-1$
			RmiServer.consolelogger.error(Messages
					.getString("ModelController.0") + e.getMessage()); //$NON-NLS-1$
		}

		return results;
	}

	/**
	 * Schreibt einen Datensatz in die Datenbank
	 * 
	 * @param entity
	 *            Entity
	 * @return true wenn Speichern funktioniert hat
	 */
	public static boolean insertData(Object entity) {
		boolean result = false;

		EntityTransaction transaction = null;
		try {
			transaction = em.getTransaction();
			transaction.begin();

			// Datensatz schreiben
			// http://www.developerfusion.com/article/84945/flush-and-clear-or-mapping-antipatterns/
			em.persist(entity);

			// (nach schreibenden Zugriffen) alle nderungen werden mit der DB
			// synchronisiert
			// und beim Commit der TX in die DB geschrieben
			// siehe hier:
			// http://www.prozesse-und-systeme.de/jpaEinleitung.html
			em.flush();

			// Daten endgltig bermitteln, flush() muss in der Transaktion
			// ablaufen
			// beim commit wird die ID des Datensatzes ermittelt
			transaction.commit();

			result = true;
		} catch (Exception e) {
			if (transaction != null && transaction.isActive()) {
				transaction.rollback();
			}

			System.out.println(""); //$NON-NLS-1$
			RmiServer.consolelogger.error(Messages
					.getString("ModelController.1") + e.getMessage()); //$NON-NLS-1$

			result = false;
		} finally {
		}

		/**
		 * Views aktualisieren
		 */
		if (result) {
			refreshEntities(entity);
		}

		return result;
	}

	/**
	 * ndert einen Datensatz in der Datenbank
	 * 
	 * @param entity
	 *            Entity
	 * @return true wenn Speichern funktioniert hat
	 */
	public static boolean changeData(Object entity) {
		boolean result = false;

		// em.setFlushMode(FlushModeType.COMMIT);

		EntityTransaction transaction = null;
		try {
			transaction = em.getTransaction();
			transaction.begin();

			// em.lock(entity, LockModeType.OPTIMISTIC);

			// Datensatz schreiben
			// http://www.binaris-informatik.de/wp-content/uploads/2011/10/EntityManager_API_JPA_1_0.pdf
			entity = em.merge(entity);

			// (nach schreibenden Zugriffen) alle nderungen werden mit der DB
			// synchronisiert
			// und beim Commit der TX in die DB geschrieben
			// siehe hier:
			// http://www.prozesse-und-systeme.de/jpaEinleitung.html
			em.flush();

			// Daten endgltig bermitteln, flush() muss in der Transaktion
			// ablaufen
			// beim commit wird die ID des Datensatzes ermittelt
			transaction.commit();

			result = true;
		} catch (Exception e) {
			if (transaction != null && transaction.isActive()) {
				transaction.rollback();
			}

			System.out.println(""); //$NON-NLS-1$
			RmiServer.consolelogger.error(Messages
					.getString("ModelController.2") + e.getMessage()); //$NON-NLS-1$

			result = false;
		} finally {
		}

		/**
		 * Views aktualisieren
		 */
		if (result) {
			refreshEntities(entity);
		}

		return result;
	}

	/**
	 * Lscht einen Datensatz in der Datenbank
	 * 
	 * @param entity
	 *            Entity
	 * @return true wenn Speichern funktioniert hat
	 */
	public static boolean deleteData(Object entity) {
		boolean result = false;

		// em.setFlushMode(FlushModeType.COMMIT);

		EntityTransaction transaction = null;
		// System.out.println(((Projekt) entity).getTitel());
		try {
			transaction = em.getTransaction();
			transaction.begin();

			// em.lock(entity, LockModeType.OPTIMISTIC);

			entity = em.merge(entity);
			em.remove(entity);

			// (nach schreibenden Zugriffen) alle nderungen werden mit der DB
			// synchronisiert
			// und beim Commit der TX in die DB geschrieben
			// siehe hier:
			// http://www.prozesse-und-systeme.de/jpaEinleitung.html
			em.flush();

			// Daten endgltig bermitteln, flush() muss in der Transaktion
			// ablaufen
			// beim commit wird die ID des Datensatzes ermittelt
			transaction.commit();

			result = true;
		} catch (Exception e) {
			if (transaction != null && transaction.isActive()) {
				transaction.rollback();
			}

			System.out.println(""); //$NON-NLS-1$
			RmiServer.consolelogger.error(Messages
					.getString("ModelController.3") + e.getMessage()); //$NON-NLS-1$

			result = false;
		} finally {
		}

		/**
		 * Views aktualisieren
		 */
		if (result) {
			refreshEntities(entity);
		}

		return result;
	}

	/**
	 * Aktualisieren der Entities
	 * 
	 * @param entity
	 *            Entity
	 */
	private static void refreshEntities(Object entity) {
		/**
		 * Cache aktualisieren
		 * http://wiki.eclipse.org/EclipseLink/Examples/JPA/Caching
		 * #How_to_refresh_the_cache
		 */
		emf.getCache().evictAll();

		/**
		 * Tabellen sollen nicht mehrfach refresht werden
		 */
		HashSet<Class> tablesHashSet = new HashSet<Class>();
		Class tempClass;

		/**
		 * Views aktualisieren
		 */
		selectData(entity.getClass());

		/**
		 * von der Tabelle referenzierte Tabellen refreshen
		 */
		for (ManagedType<?> table : tableList) {
			for (Attribute<?, ?> test : table.getAttributes()) {
				if (test.getJavaType().isAssignableFrom(entity.getClass())) {
					tempClass = table.getJavaType();
					if (!tablesHashSet.contains(tempClass)) {
						selectData(tempClass);
						tablesHashSet.add(tempClass);
					}
				}
			}
		}
	}
}
