package controller;

import java.rmi.RemoteException;
import java.rmi.server.RemoteServer;
import java.rmi.server.ServerNotActiveException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Realisierung des Observer-Patterns ber RMI
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class ObserverUpdater {
	private static final ArrayList<ClientInterface> remoteObserverList = new ArrayList<ClientInterface>();
	private static final ArrayList<String> remoteObserverListHostNames = new ArrayList<String>();
	public static String activeObserver = ""; //$NON-NLS-1$

	/**
	 * Observer hinzufgen
	 * 
	 * @param observer
	 *            Observer
	 */
	public static synchronized void addRemoteObserver(ClientInterface observer) {
		/**
		 * zurckliefern der durch JPA gemanagten Java-Klassen
		 */
		try {
			activeObserver = RemoteServer.getClientHost();
		} catch (ServerNotActiveException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		/**
		 * Observer der Liste hinzufgen
		 */
		remoteObserverList.add(observer);
		remoteObserverListHostNames.add(activeObserver);

		/**
		 * Logging des Zeitstempels
		 */
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss"); //$NON-NLS-1$
		String time = sdf.format(new Date());
		RmiServer.fileLogger.info(Messages.getString("ObserverUpdater.2") //$NON-NLS-1$
				+ time
				+ Messages.getString("ObserverUpdater.3") //$NON-NLS-1$
				+ remoteObserverListHostNames.get(remoteObserverListHostNames
						.size() - 1));
	}

	/**
	 * Observer entfernen
	 * 
	 * @param observer
	 *            Observer
	 */
	public static synchronized void removeRemoteObserver(
			ClientInterface observer) {
		int index = remoteObserverList.indexOf(observer);
		remoteObserverList.remove(observer);

		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss"); //$NON-NLS-1$
		String time = sdf.format(new Date());
		RmiServer.fileLogger
				.info(Messages.getString("ObserverUpdater.5") + time //$NON-NLS-1$
						+ Messages.getString("ObserverUpdater.6") //$NON-NLS-1$
						+ remoteObserverListHostNames.get(index));

		remoteObserverListHostNames.remove(index);
	}

	/**
	 * Observer aktualisieren
	 * 
	 * @param o
	 *            Observer
	 */
	public static synchronized void update(Class t, List<Object> o) {
		List<ClientInterface> brokenRemoteObserverList = new ArrayList<ClientInterface>();

		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss"); //$NON-NLS-1$
		String time = sdf.format(new Date());

		/**
		 * durch alle Observer iterieren
		 */
		for (ClientInterface observer : remoteObserverList) {
			try {
				activeObserver = remoteObserverListHostNames
						.get(remoteObserverList.indexOf(observer));
				// Socket socket =
				// RMISocketFactory.getSocketFactory().createSocket("",
				// Registry.REGISTRY_PORT);
				observer.update(t, o);
			} catch (Exception e) {
				System.out.println(""); //$NON-NLS-1$
				RmiServer.consolelogger
						.error(Messages.getString("ObserverUpdater.9") //$NON-NLS-1$
								+ time
								+ Messages.getString("ObserverUpdater.10") + e.getMessage()); //$NON-NLS-1$
				/**
				 * direkt entfernen ist nicht mglich weil der Iterator damit
				 * nicht klarkommen wrde
				 */
				brokenRemoteObserverList.add(observer);
			}
		}

		/**
		 * nicht mehr vorhandene Observer aus Liste entfernen
		 */
		int index = 0;
		for (ClientInterface observer : brokenRemoteObserverList) {
			index = remoteObserverList.indexOf(observer);
			remoteObserverList.remove(observer);
			remoteObserverListHostNames.remove(index);
		}

		RmiServer.fileLogger.info(Messages.getString("ObserverUpdater.11") //$NON-NLS-1$
				+ remoteObserverList.size()
				+ Messages.getString("ObserverUpdater.12") + time); //$NON-NLS-1$

		activeObserver = "";
	}

	/**
	 * beende alle Clients bevor Server beendet wird
	 */
	public static synchronized void terminateClients() {
		for (ClientInterface observer : remoteObserverList) {
			try {
				activeObserver = remoteObserverListHostNames
						.get(remoteObserverList.indexOf(observer));
				// Socket socket =
				// RMISocketFactory.getSocketFactory().createSocket("",
				// Registry.REGISTRY_PORT);
				observer.serverTerminated();
			} catch (Exception e) {
				System.out.println(""); //$NON-NLS-1$
				RmiServer.consolelogger.error(Messages
						.getString("ObserverUpdater.14") + e.getMessage()); //$NON-NLS-1$
			}
		}
		activeObserver = "";
	}
}