package controller;

import java.io.IOException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.RMISocketFactory;
import java.security.Permission;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.SimpleLayout;

/**
 * Startklasse des Servers
 * 
 * @author Thomas Kramer, eMail: th-rzv@gmx.de
 * 
 */
public class RmiServer {
	public static Controller controller = null;
	public static Registry registry = null;
	public static ServerInterfaceImpl rmiServer = null;
	public static RMISocketFactory customRMISocketFactory = null;

	public static Logger fileLogger = Logger.getRootLogger();
	public static Logger consolelogger = Logger.getLogger("ConsoleLogger"); //$NON-NLS-1$

	public static void main(String[] args) {
		int port = Registry.REGISTRY_PORT;

		/**
		 * Logging-Klasse initialisieren
		 */
		try {
			SimpleLayout layout = new SimpleLayout();
			ConsoleAppender consoleAppender = new ConsoleAppender(layout);
			consolelogger.addAppender(consoleAppender);

			FileAppender fileAppender = new FileAppender(layout, "log.txt", //$NON-NLS-1$
					false);
			fileLogger.addAppender(fileAppender);

			// ALL | DEBUG | INFO | WARN | ERROR | FATAL | OFF:
			fileLogger.setLevel(Level.INFO);
			consolelogger.setLevel(Level.INFO);

			SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss"); //$NON-NLS-1$
			String time = sdf.format(new Date());
			fileLogger.info(Messages.getString("RmiServer.3") + time); //$NON-NLS-1$
		} catch (Exception ex) {
			System.out.println(ex);
		}

		/**
		 * Hinweis ausgeben
		 */
		consolelogger.info(Messages.getString("RmiServer.4")); //$NON-NLS-1$

		/**
		 * Setze Policy-Datei mit Rechten
		 */
		System.setProperty("java.security.policy", "file:./server.policy");

		/**
		 * setze eigene SocketFactory, um Clients mit mehreren
		 * Netzwerkschnittstellen korrekt zu handlen
		 */
		try {
			customRMISocketFactory = new CustomRMISocketFactory();
			RMISocketFactory.setSocketFactory(customRMISocketFactory);
		} catch (IOException e) {
			RmiServer.consolelogger
					.error(Messages.getString("RmiServer.16") + e.getMessage()); //$NON-NLS-1$
		}

		/**
		 * Erstellen des Security-Managers fr RMI-Kommunikation
		 */
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager() {

				@Override
				public void checkPermission(Permission permission) {
				}
			});
		}

		/**
		 * starten der RMI-Registrierung entweder intern ber createRegistry
		 * oder extern ber Kommandozeile mit >>start rmiregistry -J-classpath
		 * -J"RMIServer.jar<<
		 */
		if (args.length > 0) {
			port = Integer.parseInt(args[0]);

			if (args.length > 1) {
				if (args[1].substring(0, 14).equals("createRegistry")) { //$NON-NLS-1$
					if (args[1].split("=")[1].equals("true")) { //$NON-NLS-1$ //$NON-NLS-2$
						consolelogger.info(Messages.getString("RmiServer.8") //$NON-NLS-1$
								+ port);
						try {
							registry = LocateRegistry.createRegistry(port);
						} catch (RemoteException e) {
							consolelogger.error(Messages
									.getString("RmiServer.9") //$NON-NLS-1$
									+ port);
						}
					}
				}
			}
		} else {
			consolelogger.info(Messages.getString("RmiServer.10") //$NON-NLS-1$
					+ port);
			try {
				registry = LocateRegistry.createRegistry(port);
			} catch (RemoteException e) {
				consolelogger.error(Messages.getString("RmiServer.11") //$NON-NLS-1$
						+ port);
			}
		}

		/**
		 * holen der Registrierung, auch wenn sie schon existiert
		 */
		if (registry == null) {
			consolelogger.info(Messages.getString("RmiServer.12") //$NON-NLS-1$
					+ port);
			try {
				registry = LocateRegistry.getRegistry(port);
			} catch (RemoteException e) {
				consolelogger.error(Messages.getString("RmiServer.13")); //$NON-NLS-1$
				System.exit(1);
			}

			consolelogger.info(Messages.getString("RmiServer.14")); //$NON-NLS-1$
		} else {
			consolelogger.info(Messages.getString("RmiServer.15")); //$NON-NLS-1$
		}

		boolean ok = false;

		/**
		 * Registry konnte erstellt oder geholt werden
		 */
		if (registry != null) {
			try {
				rmiServer = new ServerInterfaceImpl();
				registry.rebind(ServerInterface.SERVICE_NAME, rmiServer);

				ok = true;
			} catch (Exception e) {
				consolelogger.error(Messages.getString("RmiServer.17") //$NON-NLS-1$
						+ ServerInterface.SERVICE_NAME
						+ Messages.getString("RmiServer.18") + e.getMessage()); //$NON-NLS-1$
				consolelogger.info(Messages.getString("RmiServer.19")); //$NON-NLS-1$
			}
		}

		/**
		 * Erzeugen des Controllers wenn alles geklappt hat
		 */
		if (ok) {
			controller = new Controller();
		} else {
			consolelogger.info(Messages.getString("RmiServer.20")); //$NON-NLS-1$
			System.exit(1);
		}
	}
}